@@ -1,13 +1,8 @@
-import numbers  # noqa: E402
-try:
-    basestring  # basestring was removed in Python 3
-except NameError:
-    basestring = str
-
-# ----------------------------------------------------------------------------
+# -*- coding: utf-8 -*-
 
 
-#  ---------------------------------------------------------------------------
+import numbers  # noqa E402
+
 
 def test_ohlcv(exchange, ohlcv, symbol, now):
 
